SmartMacro = {}
if not _G["SmartMacroMasterFrame"] then
    SmartMacro.Frame = CreateFrame("Frame", "SmartMacroMasterFrame")
else
    SmartMacro.Frame = _G["SmartMacroMasterFrame"]
end
SmartMacro.Frame:RegisterEvent("PLAYER_ENTERING_WORLD")
SmartMacro.Macros = {}
SmartMacro.Setups = {}
SmartMacro.RegisteredActions = {}
SmartMacro.Localizations = {}

SmartMacro.DefaultBody = "#showtooltip\n"
SmartMacro.LastUpdate = 0
SmartMacro.RateLimit = 1.5

function SmartMacro:FindMacro(name)
    local gmc = GetNumMacros()
    local index = -1
    for i=1, gmc do
        if name == GetMacroInfo(i) then
            index = i
            break
        end
    end
    return index
end

function SmartMacro:CreateMacro(name)
    local id = self:FindMacro(name)
    if id == -1 then
        id = CreateMacro(name, "INV_MISC_QUESTIONMARK", "#showtooltip\n", nil)
    end
end

function SmartMacro:BuildLocalizations(...)
    local t = {...}
    for v,_ in pairs(t) do
        local res = C_Item.GetItemNameByID(v)
        if res then
            self.Localizations[v] = res
        end
    end
end

function SmartMacro:GetLocalName(itemId)
    local name = self.Localizations[itemId]
    if not name then
        print("not found: ", itemId)
    end
    return name
end

function SmartMacro:Register(macro)
    for k,v in pairs(macro.EventActions) do
        if not self.RegisteredActions[k] then
            self.RegisteredActions[k] = {}
            self.Frame:RegisterEvent(k)
        end
        tinsert(self.RegisteredActions[k], macro.Update)
    end
end

function SmartMacro:Update(event, ...)
    if time() - self.LastUpdate <= self.RateLimit then
        return
    end
    self.LastUpdate = time()
    for _,f in pairs(self.RegisteredActions[event]) do
        f(event, ...)
    end
end

SmartMacro.Template = {}
SmartMacro.Setup = {}

function SmartMacro.Setup.New(name, itemIDs, condition)
    local setup = setmetatable({}, SmartMacro.Setup)
    setup.Name = name
    setup.EventActions = {}
    setup.Items = {}
    setup.Loaded = false
    setup.Default = itemIDs[1]
    if condition then
        setup.Condition = condition
    else
    setup.Condition = function(id) return GetItemCount(id) > 0 end
    end
    setup.Container = ContinuableContainer:Create()
    for i, id in pairs(itemIDs) do
        local item = Item:CreateFromItemID(id)
        if i == 1 then
            setup.Default = item
        else    
            setup.Items[id] = item
        end
        setup.Container:AddContinuable(item)
    end
    setup.Container:ContinueOnLoad(function()
        setup.Loaded = true
        setup.Default.UseString = "/use " .. setup.Default:GetItemName() .. "\n"
        for id,item in pairs(setup.Items) do
            setup.Items[id].UseString = "/use " .. item:GetItemName() .. "\n"
        end
        setup.Build = function()
            local body = setup.Default.UseString
            for _, item in pairs(setup.Items) do
                if setup.Condition(item:GetItemID()) then
                    body = item.UseString .. body
                end
            end
            return body
        end
        setup.EventActions["UNIT_INVENTORY_CHANGED"] = function (unitId)
            if unitId == 'player' then
                return setup.Build()
            end
        end
        setup.EventActions["PLAYER_ENTERING_WORLD"] = setup.Build
        SmartMacro.Macros[setup.Name] = SmartMacro.Template.New(setup.Name, setup.EventActions)
    end)
    return setup
end


function SmartMacro.Template.New(name, eventActions)
    local self = setmetatable({}, SmartMacro.Template)
    self.Name = name
    SmartMacro:CreateMacro(self.Name)
    self.EventActions = eventActions
    self.Update = function(event, ...)
        local newbody = self.EventActions[event](...)
        if newbody then
            EditMacro(self.Name, nil, nil, SmartMacro.DefaultBody .. newbody, 1, 1)
        end
    end
    SmartMacro:Register(self)
    return self
end

function SmartMacro:SetupMana()
    local name = "MaximumMP"
    local ids = {22832, 32902}
    local maps = {[552]=true, [553]=true, [554]=true, [550]=true}
    SmartMacro.Setups[name] = SmartMacro.Setup.New(name, ids, function (id)
        local map, _ = select(8, GetInstanceInfo())
        return maps[map] and GetItemCount(id) > 0
    end)
end

function SmartMacro:SetupHealth()
    local maps = {[552]=true, [553]=true, [554]=true, [550]=true}
    local ids = {22829, 32905}
    local name = "MaximumHP"
    self.Setups[name] = SmartMacro.Setup.New(name, ids, function (id)
        local map, _ = select(8, GetInstanceInfo())
        return maps[map] and GetItemCount(id) > 0
    end)
end

function SmartMacro:SetupWater()
    local name = "MaximumWater"
    local ids = {27860, 22018, 34062}
    self.Setups[name] = SmartMacro.Setup.New(name, ids, nil)
end

SmartMacro.Frame:SetScript("OnEvent", function(self, event, ...)
    if InCombatLockdown() then
        return
    end
    if not SmartMacro.Setups["MaximumMP"] then
        SmartMacro:SetupMana()
    end
    if not SmartMacro.Setups["MaximumHP"] then
        SmartMacro:SetupHealth()
    end
    if not SmartMacro.Setups["MaximumWater"] then
        SmartMacro:SetupWater()
    end
    if SmartMacro.Setups["MaximumMP"].Loaded and SmartMacro.Setups["MaximumHP"].Loaded and SmartMacro.Setups["MaximumWater"].Loaded then
        SmartMacro.Frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
        SmartMacro.Frame:SetScript("OnEvent", function (self, event, ...)
            if InCombatLockdown() then
                return
            end
            SmartMacro:Update(event, ...)
        end)
        SmartMacro:Update(event, ...)
    end
end)
